function f = fun_impunk_solution(x)
% December 26, 2004
% defines the scaled difference between actual and model responses

% DEFINE PARAMETERS TO BE ESTIMATED (careful need to be changes if you calibrate other parameters)
% *******************************************************************************
bet         = 0.99;
phi         = x(1);
theta_n     = 1.0;
zeta_p      = x(2);
phi_r       = x(3);
phi_p      = x(4);
phi_x       = x(5);
rho1        = 0.8;
rho2        = 0.8;
zetaps      = (1-zeta_p)*(1-bet*zeta_p)/zeta_p;
sigma_e1    = 0.01; 
sigma_e2    = 0.01;
sigma_e3    = 0.01;
% *******************************************************************************
horizon = 20;
PERIOD  = 4;

% Solving a sticky price model of 3 equations in 3 unknowns: inflation, the gap and interest rates.
% Given that the Taylor rule is backward looking we have to defined auxiliary variables 
% for lagged inflation and lagged output

% solved with Uhlig's algorithm. 
% Endogenous state variables "x(t)": interest rate, inflation, gap
% Endogenous other variables "y(t)": lagged inflation and lagged gap
% Exogenous state variables  "z(t)": 3 equation shocks
% Switch to that notation.  Find matrices for format
% 0 = AA x(t) + BB x(t-1) + CC y(t) + DD z(t)
% 0 = E_t [ FF x(t+1) + GG x(t) + HH x(t-1) + JJ y(t+1) + KK y(t) + LL z(t+1) + MM z(t)]
% z(t+1) = NN z(t) + epsilon(t+1) with E_t [ epsilon(t+1) ] = 0,

VARNAMES = ['interest rate ',
            'inflation     ',
            'output gap    ',
            'lag inflation ',
            'lag gap       ',
            'IS shock      ',
            'AS shock      ',
            'Taylor shock  '];

 

% for i(t),  pi(t),   x(t):
AA = [ -1,    0,         0
        0,    0,         0
        0,    0,         0]; 

% for i(t-1), pi(t-1),   x(t-1):
BB = [ phi_r,   0,       0
          0,    1,       0
          0,    0,       1]; 

%Order:   lag-pi(t-1), lag-x(t-1)
CC = [    (1-phi_r)*phi_p, (1-phi_r)*phi_x
                       -1,      0
                        0,     -1];

%     e1(t)   e2(t) e3(t) 
DD = [  0,     0,    1
        0,    0,    0
        0,    0,    0]; 


% for i(t+1),  pi(t+1),   x(t+1):
FF = [  0,    1/phi,       1
        0,    bet,        0];


% for i(t),       pi(t),   x(t):
GG = [ -1/phi,    0,        -1
            0,    -1,     (phi+theta_n)*zetaps];
        
% for i(t-1),     pi(t-1),   x(t-1):

HH = [ 0,  0,  0
       0,  0,  0];
% for lag-pi(t+1), lag-x(t+1)

JJ = [ 0,  0
       0,  0];

   % for lag-pi(t), lag-x(t)

KK = [ 0,  0
       0,  0];

%     e1(t+1)   e2(t+1) e3(t+1) 
LL = [ 0,  0,  0
       0,  0,  0];

%     e1(t)   e2(t) e3(t) 

MM = [ 1,  0,  0
       0,  1,  0];

NN = zeros(3,3);
NN(1,1) = rho1;
NN(2,2) = rho2;
NN(3,3) = 0;

Sigma = [ sigma_e1^2,          0,        0
                   0, sigma_e2^2,        0
                   0,          0, sigma_e3^2];


[QQ,SS,NN,PP,RR,flag] = solving(AA,BB,CC,DD,FF,GG,HH,JJ,KK,LL,MM,NN);    
               
                   
% *********************************
% computation of impulse responses
% *********************************
[m_states,k_exog] = size(QQ);
[n_endog,k_exog]  = size(SS);
Time_axis = (0 : (horizon-1))/PERIOD;
Resp_mat = [];

% *********************************************************************
% 1 = is shock; 2 = phillips curve shock; 3 = monetary policy shock
% *********************************************************************
 shock_counter = 3;

Response = zeros(m_states+n_endog+k_exog,horizon);
Response(m_states+n_endog+shock_counter,1) = 1;
   
II_lag = [ PP, zeros(m_states,n_endog),zeros(m_states,k_exog)
           RR, zeros(n_endog, n_endog),zeros(n_endog, k_exog)
           zeros(k_exog,(m_states+n_endog)), NN                ];
              
II_contemp = eye(m_states+n_endog+k_exog) + ...
        [ zeros(m_states,(m_states+n_endog)), QQ
          zeros(n_endog, (m_states+n_endog)), SS
          zeros(k_exog,  (m_states+n_endog)), zeros(k_exog,k_exog) ];
       
Response(:,1) = II_contemp*Response(:,1);
       
for time_counter = 2 : horizon,
     Response(:,time_counter) = II_contemp*II_lag*Response(:,time_counter-1);
end;
      
response_output_gap    = Response(3,:)';
response_infl          = Response(2,:)';
response_interest_rate = Response(1,:)';
  
       
% DERIVE OBJECTIVE FUNCTION TO MINIMIZE
% ******************************************************************************************************************
data = xlsread('monresp.xls');
y_median = data(:,1);
p_median = data(:,3);
r_median = data(:,5);
se_y = data(:,2);
se_p = data(:,4);
se_r = data(:,6);
  
imp_MODEL = [response_output_gap(1:horizon);response_infl(1:horizon);response_interest_rate(1:horizon)];
imp_VAR   = [y_median(1:horizon);p_median(1:horizon);r_median(1:horizon)];

% standard error impulse responses from the data
VW=[se_y(1:horizon); se_p(1:horizon); se_r(1:horizon)];
 
impDIF    = imp_MODEL - imp_VAR;

% quadratic form to be used with generic minimization function
%impDIST   = impDIF.*VW.^{-2}*impDIF;
% normal form to be used with non-linear LS
VV=VW;
exx=-1;
impDIST   = impDIF.*(VV.^exx);

%  OBJECTIVE FUNCTION
f = impDIST;

