% estimation of the parameters of a NK model matching impulse responses to a monetary shock
% f.canova, 26-12-2004, version 1.1

% model is
% gap(t) = E(t) gap(t+1) - 1/phi (i(t)-E(t) pi(t+1)) + e(1t)  IS curve
% pi(t) = bet E(t) pi(t+1) +(phi+ theta_n) (1-zeta_p)(1-bet*zeta_p)/zeta_p gap(t)+ e(2t)  Phillips curve
% i(t)  = a_r i(t-1)+ a_x gap(t-1) +a_pi pi(t-1)+ e_(3t) policy
% e(1t) is AR(1) with persistence rho1
% e(2t) is AR(1) with persistence rho2

% the data monresp.xls contains responses and standard errors of responses computed with monte carlo
% sign restrictions are use to identify monetary shocks.
% data organized as y_median se_y p_median se_p r_median se_r

data = xlsread('monresp.xls');
y_median = data(:,1);
p_median = data(:,3);
r_median = data(:,5);
se_y = data(:,2);
se_p = data(:,4);
se_r = data(:,6);

%%%%%%%%%%%%%%%%%%%%%%%%% exercises %%%%%%%%%%%%%%%%%%%%%%%%%%
% exercise 1: use 10 or 40 horizons to estimate the parameters
% exercise 2: restrict the upper and lower limit of the estimation: cut intervals by half
% exercise 3: add theta_n to the calibrated parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Parameters for estimation/plot
% *******************************************************************************
horizon = 20;  % number of periods in impulse responses
PERIOD  = 4;   % number of points  in each year
NVAR    = 3;   % number of variables in the system
% *******************************************************************************
% Parameters not estimated (not-identificable)
% *******************************************************************************
sigma_e1    = 0.01; %standard errors of the three shocks
sigma_e2    = 0.01;
sigma_e3    = 0.01;
bet         = 0.99;
rho1        = 0.8;
rho2        = 0.8;
% *******************************************************************************
% Set initial values on parameters to be estimated
% *******************************************************************************
x0    = [2.0;  2.0;    0.75 ;  0.8;  1.5;  0.5 ];
%       [phi;  theta_n;zeta_p; phi_r; phi_p ; phi_x ];
% *******************************************************************************
% Set constraints (upper and lower limit) on the parameters to be estimated
% *******************************************************************************
lb    = [0.1;   0.1;   0.10;  0.00; 1.01;  -1.0];
ub    = [10.0;  10.0;  0.98;  1.00; 5.0;    1.0];
% *******************************************************************************

% Find minimum distance using non-linear least square routine
% ***************************************************************************************************************
options=optimset('MaxFunEvals',4000,'MaxIter',2000);
[x,resnorm,residual,exitflag,OUTPUT,LAMBDA,JACOBIAN]=lsqnonlin(@fun_impunk,x0,lb,ub,options);
% ***************************************************************************************************************

% diagonal variance matrix of impulse responses
% *******************************************************************************************
VCVimp = zeros(NVAR*horizon, NVAR*horizon);

for j=1,horizon;
    VCVimp(j,j)= se_y(j)^2;
    VCVimp(j+horizon,j+horizon)= se_p(j)^2;
    VCVimp(j+2*horizon,j+2*horizon)= se_p(j)^2;
end;    

% Standard errors of parameters
% ************************************************************************************************************************
VMDE   = inv(JACOBIAN'*JACOBIAN)*JACOBIAN'*VCVimp*JACOBIAN*inv(JACOBIAN'*JACOBIAN);
STDMDE = (diag(VMDE)).^(1/2);
% *************************************************************************************************************************


% Results
% *******************************************************************************
disp(' ')
disp(' ')
disp('************************************')
disp('CALIBRATED PARAMETERS')
disp('************************************')
disp('standard error shocks')
disp([' sigma_e1    = ' num2str(sigma_e1) ''])
disp([' sigma_e2    = ' num2str(sigma_e2) ''])
disp([' sigma_e3    = ' num2str(sigma_e3) ''])
disp([' bet         = ' num2str(bet) ''])
disp([' rho1        = ' num2str(rho1) ''])
disp([' rho2        = ' num2str(rho2) ''])
disp('************************************')
disp('ESTIMATED PARAMETERS')
disp('************************************')
disp([' phi    = ' num2str(x(1)) ''])
disp(['          (' num2str(STDMDE(1)) ')'])
disp([' theta_n      = ' num2str(x(2)) ''])
disp(['          (' num2str(STDMDE(2)) ')'])
disp([' zeta_p      = ' num2str(x(3)) ''])
disp(['          (' num2str(STDMDE(3)) ')'])
disp([' phi_r     = ' num2str(x(4)) ''])
disp(['          (' num2str(STDMDE(4)) ')'])
disp([' phi_pi    = ' num2str(x(5)) ''])
disp(['          (' num2str(STDMDE(5)) ')'])
disp([' phi_x      = ' num2str(x(6)) ''])
disp(['          (' num2str(STDMDE(6)) ')'])
disp('------------------------------------')
disp(['resnorm  = ' num2str(resnorm) ''])
% *******************************************************************************


% calculate impulse responses with the estimated parameters

phi         = x(1);
theta_n     = x(2);
zeta_p      = x(3);
phi_r       = x(4);
phi_p      = x(5);
phi_x       = x(6);
zetaps      = (1-zeta_p)*(1-bet*zeta_p)/zeta_p;
sigma_e1    = 0.01; 
sigma_e2    = 0.01;
sigma_e3    = 0.01;
bet         = 0.99;
rho1        = 0.8;
rho2        =0.8;

 
% solved with Uhlig's algorithm. 
% Endogenous state variables "x(t)": interest rate, inflation, gap
% Endogenous other variables "y(t)": lagged inflation and lagged gap
% Exogenous state variables  "z(t)": 3 equation shocks
% Switch to that notation.  Find matrices for format
% 0 = AA x(t) + BB x(t-1) + CC y(t) + DD z(t)
% 0 = E_t [ FF x(t+1) + GG x(t) + HH x(t-1) + JJ y(t+1) + KK y(t) + LL z(t+1) + MM z(t)]
% z(t+1) = NN z(t) + epsilon(t+1) with E_t [ epsilon(t+1) ] = 0,

VARNAMES = ['interest rate ',
            'inflation     ',
            'output gap    ',
            'lag inflation ',
            'lag gap       ',
            'IS shock      ',
            'AS shock      ',
            'Taylor shock  '];

 
zeteps = (1-zeta_p)*(1-bet*zeta_p)/zeta_p;

% for i(t),  pi(t),   x(t):
AA = [ -1,    0,         0
        0,    0,         0
        0,    0,         0]; 

% for i(t-1), pi(t-1),   x(t-1):
BB = [ phi_r,   0,       0
          0,    1,       0
          0,    0,       1]; 

%Order:   lag-pi(t-1), lag-x(t-1)
CC = [    (1-phi_r)*phi_p, (1-phi_r)*phi_x
                       -1,      0
                        0,     -1];

%     e1(t)   e2(t) e3(t) 
DD = [  0,     0,    1
        0,    0,    0
        0,    0,    0]; 


% for i(t+1),  pi(t+1),   x(t+1):
FF = [  0,    1/phi,       1
        0,    bet,        0];


% for i(t),       pi(t),   x(t):
GG = [ -1/phi,    0,        -1
            0,    -1,     (phi+theta_n)*zetaps];
        
% for i(t-1),     pi(t-1),   x(t-1):

HH = [ 0,  0,  0
       0,  0,  0];
% for lag-pi(t+1), lag-x(t+1)

JJ = [ 0,  0
       0,  0];

   % for lag-pi(t), lag-x(t)

KK = [ 0,  0
       0,  0];

%     e1(t+1)   e2(t+1) e3(t+1) 
LL = [ 0,  0,  0
       0,  0,  0];

%     e1(t)   e2(t) e3(t) 

MM = [ 1,  0,  0
       0,  1,  0];

NN = zeros(3,3);
NN(1,1) = rho1;
NN(2,2) = rho2;
NN(3,3) = 0;

Sigma = [ sigma_e1^2,          0,        0
                   0, sigma_e2^2,        0
                   0,          0, sigma_e3^2];

               
 
[QQ,SS,NN,PP,RR,flag] = solving(AA,BB,CC,DD,FF,GG,HH,JJ,KK,LL,MM,NN);    

                   
               
 

% *********************************
% computation of impulse responses
% *********************************

[m_states,k_exog] = size(QQ);
[n_endog,k_exog]  = size(SS);
Time_axis = (0 : (horizon-1))/PERIOD;
Resp_mat = [];

response_output_gap = zeros(3,horizon);
response_infl = zeros(3,horizon);
response_interest_rate = zeros(3,horizon);
 
% *********************************************************************
% 1 = is shock; 2 = phillips curve shock; 3 = monetary policy shock
% *********************************************************************
for i = 1 : NVAR,
shock_counter = i;

Response = zeros((m_states+n_endog+k_exog),horizon);
Response(m_states+n_endog+shock_counter,1) = 1;

   
II_lag = [ PP, zeros(m_states,n_endog),zeros(m_states,k_exog)
           RR, zeros(n_endog, n_endog),zeros(n_endog, k_exog)
           zeros(k_exog,(m_states+n_endog)), NN                ];
              
II_contemp = eye(m_states+n_endog+k_exog) + ...
        [ zeros(m_states,(m_states+n_endog)), QQ
          zeros(n_endog, (m_states+n_endog)), SS
          zeros(k_exog,  (m_states+n_endog)), zeros(k_exog,k_exog) ];
       
Response(:,1) = II_contemp*Response(:,1);
       
for time_counter = 2 : horizon,
     Response(:,time_counter) = II_contemp*II_lag*Response(:,time_counter-1);
end;
      
response_output_gap(i,:)    = Response(3,:);
response_infl(i,:)          = Response(2,:);
response_interest_rate(i,:) = Response(1,:);
 

% response of price level
%response_price = zeros(horizon,1);
%response_price(1,1) = response_infl(1);
%for i = 2:horizon
%	response_price(i,1) = response_price(i-1,1)+response_infl(i,1);
%end 

%zero_ax = zeros(horizon,1);
end;



%saving results
parmod = x;
refun = resnorm;

%graphing impulse responses
figure
subplot(3,3,1);
plot(response_output_gap(1,:),'-');
title('IS shocks');
ylabel('Output gap');
subplot(3,3,4);
plot(response_infl(1,:),'-');
ylabel('inflation');
subplot(3,3,7);
plot(response_interest_rate(1,:),'-');
ylabel('interest rate');
subplot(3,3,2);
plot(response_output_gap(2,:),'-');
title('Phillips shocks');
subplot(3,3,5);
plot(response_infl(2,:),'-');
subplot(3,3,8);
plot(response_interest_rate(2,:),'-');
subplot(3,3,3);
plot(response_output_gap(3,:),'-');
title('Monetary shocks');
subplot(3,3,6);
plot(response_infl(3,:),'-');
subplot(3,3,9);
plot(response_interest_rate(3,:),'-');






 